/* * Copyright (c) 2015, Freescale Semiconductor, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of Freescale Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
#include "EM7164SU16.h"
#include "flexbus.h"
#include "common.h"


#define SRAM_DEBUG		0
#if ( SRAM_DEBUG == 1 )
#include <stdio.h>
#define SRAM_TRACE	printf
#else
#define SRAM_TRACE(...)
#endif

//SRAMʼ
void SRAM_Init(void)
{
    /* config SRAM and LCD pinmux */
    SIM->SCGC5 |= (SIM_SCGC5_PORTA_MASK | SIM_SCGC5_PORTB_MASK | SIM_SCGC5_PORTC_MASK | SIM_SCGC5_PORTD_MASK | SIM_SCGC5_PORTE_MASK);
    PORTD->PCR[6] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD0
    PORTD->PCR[5] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD1
    PORTD->PCR[4] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD2
    PORTD->PCR[3] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD3
    PORTD->PCR[2] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD4
    PORTC->PCR[10] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD5
    PORTC->PCR[9] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD6
    PORTC->PCR[8] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD7 
    PORTC->PCR[7] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD8 
    PORTC->PCR[6] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD9 
    PORTC->PCR[5] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD10
    PORTC->PCR[4] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD11
    PORTC->PCR[2] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD12
    PORTC->PCR[1] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD13
    PORTC->PCR[0] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD14
    PORTB->PCR[18] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD15
    PORTB->PCR[17] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD16
    PORTB->PCR[16] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD17
    PORTB->PCR[11] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD18
    PORTB->PCR[10] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD19
    PORTB->PCR[9] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD20
    PORTB->PCR[19] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//RD
    PORTC->PCR[11] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//RW
    PORTC->PCR[3] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//CLKOUT
    PORTD->PCR[0] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//ALE
    PORTD->PCR[1] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//CS0
    PORTC->PCR[18] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//CS2
    PORTC->PCR[17] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//BE31_24
    PORTC->PCR[16] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//BE23_16
    
    /* enable flexbus */
    FLEXBUS_InitTypeDef FLEXBUS_InitStruct;
    FLEXBUS_InitStruct.ADSpaceMask = kFLEXBUS_ADSpace_1MByte;
    FLEXBUS_InitStruct.autoAckMode = kFLEXBUS_AutoAckEnable;
    FLEXBUS_InitStruct.CSn = kFLEXBUS_CS2;
    FLEXBUS_InitStruct.dataAlignMode = kFLEXBUS_DataRightAligned;
    FLEXBUS_InitStruct.dataWidth = kFLEXBUS_PortSize_16Bit;
    FLEXBUS_InitStruct.baseAddress = SRAM_ADDRESS_BASE;
    FLEXBUS_InitStruct.ByteEnableMode = kFLEXBUS_BE_AssertedReadWrite;
    FLEXBUS_InitStruct.div = 1;
    FLEXBUS_Init(&FLEXBUS_InitStruct);

    /* advanced config */
    FLEXBUS_AdvancedConfigTypeDef config;
    config.kFLEXBUS_brustWriteEnable = false;
    config.kFLEXBUS_brustReadEnable = false;
    config.kFLEXBUS_EXTS = true;
    config.kFLEXBUS_ASET = 1;
    config.kFLEXBUS_RDAH = 1;
    config.kFLEXBUS_WRAH = 1;
    config.kFLEXBUS_WS = 2;
    FLEXBUS_AdvancedConfig(FLEXBUS_InitStruct.CSn, &config);
    
    /* config Flexbus SRAM pinmux */
    FLEXBUS_PortMuxConfig(kFLEXBUS_CSPMCR_Group2, kFLEXBUS_CSPMCR_GROUP2_BE_31_24);
    FLEXBUS_PortMuxConfig(kFLEXBUS_CSPMCR_Group3, kFLEXBUS_CSPMCR_GROUP3_BE_23_16);
    FLEXBUS_PortMuxConfig(kFLEXBUS_CSPMCR_Group4, kFLEXBUS_CSPMCR_GROUP4_CS2);
}


uint32_t SRAM_SelfTest(void)
{
    uint32_t i;
    uint32_t address,size;
    address = SRAM_ADDRESS_BASE;
    size = SRAM_SIZE;
    SRAM_TRACE("memtest,address: 0x%08X size: 0x%08X\r\n", address, size);
    
    /**< 8bit test */
    {
        uint8_t * p_uint8_t = (uint8_t *)address;
        for(i=0; i<size/sizeof(uint8_t); i++)
        {
            *p_uint8_t++ = (uint8_t)i;
        }

        p_uint8_t = (uint8_t *)address;
        for(i=0; i<size/sizeof(uint8_t); i++)
        {
            if( *p_uint8_t != (uint8_t)i )
            {
                SRAM_TRACE("8bit test fail @ 0x%08X\r\nsystem halt!!!!!",(uint32_t)p_uint8_t);
                return 1;
            }
            p_uint8_t++;
        }
        SRAM_TRACE("8bit test pass!!\r\n");
    }
    
    /**< 16bit test */
    {
        uint16_t * p_uint16_t = (uint16_t *)address;
        for(i=0; i<size/sizeof(uint16_t); i++)
        {
            *p_uint16_t++ = (uint16_t)i;
        }

        p_uint16_t = (uint16_t *)address;
        for(i=0; i<size/sizeof(uint16_t); i++)
        {
            if( *p_uint16_t != (uint16_t)i )
            {
                SRAM_TRACE("16bit test fail @ 0x%08X\r\nsystem halt!!!!!",(uint32_t)p_uint16_t);
                return 2;
            }
            p_uint16_t++;
        }
        SRAM_TRACE("16bit test pass!!\r\n");
    }

    /**< 32bit test */
    {
        uint32_t * p_uint32_t = (uint32_t *)address;
        for(i=0; i<size/sizeof(uint32_t); i++)
        {
            *p_uint32_t++ = (uint32_t)i;
        }

        p_uint32_t = (uint32_t *)address;
        for(i=0; i<size/sizeof(uint32_t); i++)
        {
            if( *p_uint32_t != (uint32_t)i )
            {
                SRAM_TRACE("32bit test fail @ 0x%08X\r\nsystem halt!!!!!",(uint32_t)p_uint32_t);
                return 3;
            }
            p_uint32_t++;
        }
        SRAM_TRACE("32bit test pass!!\r\n");
    }

    /**< 32bit Loopback test */
    {
        uint32_t * p_uint32_t = (uint32_t *)address;
        for(i=0; i<size/sizeof(uint32_t); i++)
        {
            *p_uint32_t  = (uint32_t)p_uint32_t;
            *p_uint32_t++;
        }

        p_uint32_t = (uint32_t *)address;
        for(i=0; i<size/sizeof(uint32_t); i++)
        {
            if( *p_uint32_t != (uint32_t)p_uint32_t )
            {
                SRAM_TRACE("32bit Loopback test fail @ 0x%08X", (uint32_t)p_uint32_t);
                SRAM_TRACE(" data:0x%08X \r\n", (uint32_t)*p_uint32_t);
                return 4;
            }
            p_uint32_t++;
        }
        SRAM_TRACE("32bit Loopback test pass!!\r\n");
    }
    return  0;
}





