/* * Copyright (c) 2015, Freescale Semiconductor, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of Freescale Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
#include "lcd.h"
#include "gpio.h"
#include "flexbus.h"
#include "common.h"

#define LCD_DEBUG		1
#if ( LCD_DEBUG == 1 )
#define LCD_TRACE	printf
#else
#define LCD_TRACE(...)
#endif

//   Modified 
#define LCD_RS_HIGH()        GPIOA_PSOR |=(1<<14)
#define LCD_RS_LOW()         GPIOA_PCOR |=(1<<14)
//   Modified 


uint32_t lcd_id;
static void write_reg(uint16_t addr, uint16_t val)
{
    WMLCDCOM(addr);
    WMLCDDATA(val);
}

static uint16_t read_reg(uint16_t addr)
{
    WMLCDCOM(addr);
    return LCD_DATA_ADDRESS;
}

uint16_t lcd_get_id(void)
{
    return read_reg(0x00);
}

static uint16_t LCD_BGR2RGB(uint16_t c)
{
	uint16_t  r,g,b,rgb;   
	b=(c>>0)&0x1f;
	g=(c>>5)&0x3f;
	r=(c>>11)&0x1f;	 
	rgb=(b<<11)+(g<<5)+(r<<0);		 
	return(rgb);
} 


int lcd_read_pixel(int x, int y)
{
    int value;
    write_reg(0x0020, x);
    write_reg(0x0021, y);
    WMLCDCOM(0x0022);
    value = LCD_DATA_ADDRESS;
    WMLCDCOM(0x0022);
    value = LCD_DATA_ADDRESS;
    return LCD_BGR2RGB(value);
}

void lcd_clear(int c)
{
	int i;
    lcd_set_window(0, 0, LCD_X_MAX, LCD_Y_MAX);
	WMLCDCOM(0x22);
	for(i = 0; i < (LCD_X_MAX * LCD_Y_MAX); i++)
	{
		WMLCDDATA(c);	   
	}
}

void lcd_write_gram(uint16_t *buf, int len)
{
    WMLCDCOM(0x0022);
    while(len--)
    {
        WMLCDDATA(*buf++);
    }
}
void lcd_write_pixel(int x, int y, int c)
{
    switch(lcd_id)
    {
        case 0x9320:
            write_reg(0x0020, x);
            write_reg(0x0021, y);
            WMLCDCOM(0x0022);
            WMLCDDATA(c);
            break;
        case 0x8989:
            write_reg(0x004e,x);        
            write_reg(0x004f,y);  
            WMLCDCOM(0x0022);
            WMLCDDATA(c);
            break;
        default:
            break;   
    }
  
}

void lcd_set_window(int x, int y, int xlen, int ylen)
{

    switch(lcd_id)
    {
        case 0x9320:
            write_reg(0x0020, x);
            write_reg(0x0021, y);
            write_reg(0x0050, x);
            write_reg(0x0052, y);
            write_reg(0x0051, x + xlen - 1);
            write_reg(0x0053, y + ylen - 1);  
            break;        
        case 0x8989:
            write_reg(0x004e,x);       
            write_reg(0x004f,y);  
            write_reg(0x0044, (x|((x + xlen - 1)<<8)));
            write_reg(0x0045, y);
            write_reg(0x0046, y + ylen - 1);   
            break;
        default:
            break;   
    }    
} 

void lcd_hline(int xs, int xe, int y, int c)
{
    switch(lcd_id)
    {
        case 0x9320:
            write_reg(0x03,(1<<5)|(1<<4)|(0<<3)|(1<<12));
            write_reg(0x0020, xs);
            write_reg(0x0021, y);
            WMLCDCOM(0x0022);
            while(xs < xe)
            {
                WMLCDDATA(c);
                xs++;
            }
          //  write_reg(0x03,(1<<5)|(1<<4)|(0<<3)|(1<<12));
            break;
        case 0x8989:
            write_reg(0x11,(1<<5)|(1<<4)|(0<<3));
            write_reg(0x004e, xs);
            write_reg(0x004f, y);
            WMLCDCOM(0x0022);
            while(xs < xe)
            {
            WMLCDDATA(c);
            xs++;
            }
            write_reg(0x11,(1<<5)|(1<<4)|(0<<3));
            break;
        default:
            break;   
    }
   
}


void lcd_vline(int ys, int ye, int x, int c)
{
    
    switch(lcd_id)
    {
        case 0x9320:
            write_reg(0x03,(0<<5)|(0<<4)|(1<<3)|(1<<12));
            write_reg(0x0020, x);
            write_reg(0x0021, ys);
            WMLCDCOM(0x0022);
            while(ys < ye)
            {
                WMLCDDATA(c);
                ys++;
            }
           // write_reg(0x03,(1<<5)|(1<<4)|(0<<3)|(1<<12));
            break;
        case 0x8989:
            write_reg(0x11,(0<<5)|(0<<4)|(1<<3));
            write_reg(0x004e, x);
            write_reg(0x004f, ys);
            WMLCDCOM(0x0022);
            while(ys < ye)
            {
            WMLCDDATA(c);
            ys++;
            }
          // write_reg(0x11,(1<<5)|(1<<4)|(0<<3));
            break;
        default:
            break;   
    }
}

  
static inline void lcdc_send_cmd(unsigned char cmd)
{
    LCD_RS_LOW();
    LCD_COMMAND_ADDRESS  = cmd;
}

static inline void lcdc_send_data(unsigned short dat)
{	
    LCD_RS_HIGH();
    LCD_DATA_ADDRESS  = dat;
}

void LCDC_Fill_Color(unsigned short color) 
{
    unsigned int i;
    
    lcdc_send_cmd(0x2A);
    lcdc_send_data(0);
    lcdc_send_data(0);;	
    
    lcdc_send_cmd(0x2B);
    lcdc_send_data(0);
    lcdc_send_data(0);
    
    lcdc_send_cmd(0x2c);
    
    for(i = 0; i < (320*240); i++)
        lcdc_send_data(color);
}

void LCDC_Set_Window(unsigned int x, unsigned int y, unsigned int w, unsigned int h) 
{
    lcdc_send_cmd(0x2A);
    lcdc_send_data(x>>8);
    lcdc_send_data(x&0xFF);
    lcdc_send_data((x+w-1)>>8);
    lcdc_send_data((x+w-1)&0xFF);	
    
    lcdc_send_cmd(0x2B);
    lcdc_send_data(y>>8);
    lcdc_send_data(y&0xFF);
    lcdc_send_data((y+h-1)>>8);
    lcdc_send_data((y+h-1)&0xFF);
}

void LCDC_Bmp(unsigned int x, unsigned int y, unsigned int w, unsigned int h, unsigned short *bmp) 
{
    unsigned int i;
    unsigned short *bmpdata_ptr = (unsigned short *)bmp;
    
    LCDC_Set_Window(x, y, w, h);
    lcdc_send_cmd(0x2c);
    
    for(i = 0; i < (w*h); i++)
    {
        lcdc_send_data(*bmpdata_ptr++); 
    }
}

int lcd_init(void)
{
    int ret;
    ret = 1;

    /* config SRAM and LCD pinmux */
    SIM->SCGC5 |= (SIM_SCGC5_PORTA_MASK | SIM_SCGC5_PORTB_MASK | SIM_SCGC5_PORTC_MASK | SIM_SCGC5_PORTD_MASK | SIM_SCGC5_PORTE_MASK);
    PORTD->PCR[6] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD0
    PORTD->PCR[5] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD1
    PORTD->PCR[4] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD2
    PORTD->PCR[3] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD3
    PORTD->PCR[2] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD4
    PORTC->PCR[10] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD5
    PORTC->PCR[9] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD6
    PORTC->PCR[8] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD7 
    PORTC->PCR[7] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD8 
    PORTC->PCR[6] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD9 
    PORTC->PCR[5] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD10
    PORTC->PCR[4] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD11
    PORTC->PCR[2] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD12
    PORTC->PCR[1] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD13
    PORTC->PCR[0] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD14
    PORTB->PCR[18] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD15
    PORTB->PCR[17] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD16
    PORTB->PCR[16] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD17
    PORTB->PCR[11] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD18
    PORTB->PCR[10] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD19
    PORTB->PCR[9] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//AD20
    PORTB->PCR[19] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//RD
    PORTC->PCR[11] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//RW
    PORTC->PCR[3] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//CLKOUT
    PORTD->PCR[0] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//ALE
    PORTD->PCR[1] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//CS0
    PORTC->PCR[18] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//CS2
    PORTC->PCR[17] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//BE31_24
    PORTC->PCR[16] = PORT_PCR_MUX(5)|PORT_PCR_DSE_MASK;//BE23_16
    
      
    
    /* enable flexbus */
    FLEXBUS_InitTypeDef FLEXBUS_InitStruct;
    FLEXBUS_InitStruct.ADSpaceMask = kFLEXBUS_ADSpace_1MByte;
    FLEXBUS_InitStruct.autoAckMode = kFLEXBUS_AutoAckEnable;
    FLEXBUS_InitStruct.CSn = kFLEXBUS_CS0;
    FLEXBUS_InitStruct.dataAlignMode = kFLEXBUS_DataRightAligned;
    FLEXBUS_InitStruct.dataWidth = kFLEXBUS_PortSize_16Bit;
    FLEXBUS_InitStruct.baseAddress = LCD_BASE;
    FLEXBUS_InitStruct.ByteEnableMode = kFLEXBUS_BE_AssertedReadWrite;
    FLEXBUS_InitStruct.div = 3;
    FLEXBUS_Init(&FLEXBUS_InitStruct);

    /* config Flexbus SRAM pinmux */
    FLEXBUS_PortMuxConfig(kFLEXBUS_CSPMCR_Group2, kFLEXBUS_CSPMCR_GROUP2_BE_31_24);
    FLEXBUS_PortMuxConfig(kFLEXBUS_CSPMCR_Group3, kFLEXBUS_CSPMCR_GROUP3_BE_23_16);
    FLEXBUS_PortMuxConfig(kFLEXBUS_CSPMCR_Group4, kFLEXBUS_CSPMCR_GROUP4_CS2);
    
    /* advanced config */
    FLEXBUS_AdvancedConfigTypeDef config;
    config.kFLEXBUS_brustWriteEnable = false;
    config.kFLEXBUS_brustReadEnable = false;
    config.kFLEXBUS_EXTS = true;
    config.kFLEXBUS_ASET = 2;
    config.kFLEXBUS_RDAH = 2;
    config.kFLEXBUS_WRAH = 2;
    config.kFLEXBUS_WS = 0x6;
    FLEXBUS_AdvancedConfig(FLEXBUS_InitStruct.CSn, &config);
    

// Modified
    PORTC->PCR[17] = PORT_PCR_MUX(1)|PORT_PCR_SRE_MASK;    //PTC17--WR
    GPIOC_PDDR |=(1<<17);
    GPIOC_PCOR |=(1<<17);
    
    PORTA->PCR[14] = PORT_PCR_MUX(1);                      //PTA14--RS
    GPIOA_PDDR |=(1<<14);
// Modified
    
    DelayMs(100);
    lcd_id = lcd_get_id();

    lcdc_send_cmd(0xCF);    //Power control B
    lcdc_send_data(0x00); 
    lcdc_send_data(0xC1); 
    lcdc_send_data(0X30); 
    
    lcdc_send_cmd(0xED);    //Power on sequence control
    lcdc_send_data(0x64); 
    lcdc_send_data(0x03); 
    lcdc_send_data(0X12); 
    lcdc_send_data(0X81); 
    
    lcdc_send_cmd(0xE8);    //Driver timing control A
    lcdc_send_data(0x85); 
    lcdc_send_data(0x10); 
    lcdc_send_data(0x7A); 
    
    lcdc_send_cmd(0xCB);    //Power control A
    lcdc_send_data(0x39); 
    lcdc_send_data(0x2C); 
    lcdc_send_data(0x00); 
    lcdc_send_data(0x34); 
    lcdc_send_data(0x02); 
    
    lcdc_send_cmd(0xF7);    //Pump ratio control
    lcdc_send_data(0x20); 
    
    lcdc_send_cmd(0xEA);    //Driver timing control B
    lcdc_send_data(0x00); 
    lcdc_send_data(0x00); 
    
    lcdc_send_cmd(0xC0);    //Power control 
    lcdc_send_data(0x1B);   //VRH[5:0]  1B
    
    lcdc_send_cmd(0xC1);    //Power control 
    lcdc_send_data(0x01);   //SAP[2:0];BT[3:0] 
    
    lcdc_send_cmd(0xC5);    //VCM control 
    lcdc_send_data(0x45);   //3F
    lcdc_send_data(0x25);   //3C
    
    lcdc_send_cmd(0xC7);    //VCM control2 
    lcdc_send_data(0XB7);   //b7
    
    lcdc_send_cmd(0x36);    // Memory Access Control 
    lcdc_send_data(0x28); 
    
    lcdc_send_cmd(0x3A);    //Pixel Format Set
    lcdc_send_data(0x55); 
    
    lcdc_send_cmd(0xB1);    //Frame Rate Control
    lcdc_send_data(0x00);   
    lcdc_send_data(0x1A); 
    
    lcdc_send_cmd(0xB6);    // Display Function Control 
    lcdc_send_data(0x0A); 
    lcdc_send_data(0x82); 
    
    lcdc_send_cmd(0xF2);    // 3Gamma Function Disable 
    lcdc_send_data(0x00); 
    
    lcdc_send_cmd(0x26);    //Gamma curve selected 
    lcdc_send_data(0x01); 
    
    lcdc_send_cmd(0xE0);    //Set Gamma 
    lcdc_send_data(0x0F); 
    lcdc_send_data(0x2A); 
    lcdc_send_data(0x28); 
    lcdc_send_data(0x08); 
    lcdc_send_data(0x0E); 
    lcdc_send_data(0x08); 
    lcdc_send_data(0x54); 
    lcdc_send_data(0XA9); 
    lcdc_send_data(0x43); 
    lcdc_send_data(0x0A); 
    lcdc_send_data(0x0F); 
    lcdc_send_data(0x00); 
    lcdc_send_data(0x00); 
    lcdc_send_data(0x00); 
    lcdc_send_data(0x00); 
    
    lcdc_send_cmd(0XE1);    //Set Gamma 
    lcdc_send_data(0x00); 
    lcdc_send_data(0x15); 
    lcdc_send_data(0x17); 
    lcdc_send_data(0x07); 
    lcdc_send_data(0x11); 
    lcdc_send_data(0x06); 
    lcdc_send_data(0x2B); 
    lcdc_send_data(0x56); 
    lcdc_send_data(0x3C); 
    lcdc_send_data(0x05); 
    lcdc_send_data(0x10); 
    lcdc_send_data(0x0F); 
    lcdc_send_data(0x3F); 
    lcdc_send_data(0x3F); 
    lcdc_send_data(0x0F); 
    
    lcdc_send_cmd(0x2A);
    lcdc_send_data(0x00);
    lcdc_send_data(0x00);
    lcdc_send_data(0x01);
    lcdc_send_data(0x3F);	
    
    lcdc_send_cmd(0x2B);
    lcdc_send_data(0x00);
    lcdc_send_data(0x00);
    lcdc_send_data(0x00);
    lcdc_send_data(0xEF);
    
    lcdc_send_cmd(0x11);   //Exit Sleep
    DelayMs(15);
    lcdc_send_cmd(0x29);   //display on
    DelayMs(15);
    lcdc_send_cmd(0x2C);
    LCDC_Fill_Color(RED);

    
    lcd_clear(BLACK);
    return ret;
}

void GUI_DrawPixel(int color, int x, int y)
{
    switch(lcd_id)
    {
        case 0x9320:
            write_reg(0x0020, x);
            write_reg(0x0021, y);
            WMLCDCOM(0x0022);
            WMLCDDATA(color);
            break;
        case 0x8989:
            write_reg(0x004e,x);        
            write_reg(0x004f,y);  
            WMLCDCOM(0x0022);
            WMLCDDATA(color);
            break;
        default:
            break;   
    }
    
}

void GUI_DriverInit(void)
{
    lcd_init();
}
