/* * Copyright (c) 2015, Freescale Semiconductor, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of Freescale Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
  
#ifndef __CH_LIB_CAN_H__
#define __CH_LIB_CAN_H__

#ifdef __cplusplus
 extern "C" {
#endif

#include <stdint.h>
#include "common.h"  

//!< hardware instances
#define HW_CAN0  (0x00U)  //CAN0ģ
#define HW_CAN1  (0x01U)  //CAN1ģ  
     
//CANٶѡ
typedef enum 
{
	kCAN_25K,
	kCAN_50K,
	kCAN_100K,
	kCAN_125K,
	kCAN_250K,
	kCAN_500K,
	kCAN_1000K,
}CAN_Baudrate_Type;

typedef struct
{
    uint32_t                instance;  //CANͨŶ˿ HW_CAN0/HW_CAN1
    CAN_Baudrate_Type       baudrate;  //CANͨٶ
}CAN_InitTypeDef;


//!< CAN QuickInit macro
#define CAN1_TX_PE24_RX_PE25   (0xB0A1U)  //CAN1ģ ΪPTE24 ΪPTE25
#define CAN0_TX_PA12_RX_PA13   (0x9880U)  //CAN0ģ ΪPTA12 ΪPTA13
#define CAN0_TX_PB18_RX_PB19   (0xA488U)  //CAN0ģ ΪPTB18 ΪPTB19
#define CAN1_TX_PC17_RX_PC16   (0xA091U)  //CAN1ģ ΪPTC17 ΪPTC16


/*!< жϼDMA */
typedef enum
{
    kCAN_IT_Tx_Disable,  //رշж
    kCAN_IT_Rx_Disable,  //رսж
    kCAN_IT_Tx,          //ж
    kCAN_IT_RX,          //ж
}CAN_ITDMAConfig_Type;

/*!< CAN ص */
typedef void (*CAN_CallBackType)(void);

//!< API functions
void CAN_SetRxFilterMask(uint32_t instance, uint32_t mb, uint32_t mask);
uint32_t CAN_WriteData(uint32_t instance, uint32_t mb, uint32_t id, uint8_t* buf, uint8_t len);
uint32_t CAN_WriteRemote(uint32_t instance, uint32_t mb, uint32_t id, uint8_t len);
uint32_t CAN_ReadData(uint32_t instance, uint32_t mb, uint32_t *id, uint8_t *buf, uint8_t *len);
uint32_t CAN_QuickInit(uint32_t CANxMAP, CAN_Baudrate_Type baudrate);
void CAN_ITDMAConfig(uint32_t instance, uint32_t mb, CAN_ITDMAConfig_Type config);
void CAN_SetRxMB(uint32_t instance, uint32_t mb, uint32_t id);
void CAN_CallbackInstall(uint32_t instance, CAN_CallBackType AppCBFun);
void CAN_Init(CAN_InitTypeDef* CAN_InitStruct);

#ifdef __cplusplus
}
#endif

#endif
