/* * Copyright (c) 2015, Freescale Semiconductor, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of Freescale Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
	
#ifndef __CH_LIB_CRC_H__
#define __CH_LIB_CRC_H__

#include <stdint.h>
#include <stdbool.h>


typedef enum
{
    kCRCNoTranspose    = 0U, /*!< No transposition. @internal gui name="No Transpose" */
    kCRCTransposeBits  = 1U, /*!< Bits in bytes are transposed; bytes are not transposed. @internal gui name="Transpose Bits" */
    kCRCTransposeBoth  = 2U, /*!< Both bits in bytes and bytes are transposed. @internal gui name="Transpose Bits in Bytes and Bytes" */
    kCRCTransposeBytes = 3U  /*!< Only bytes are transposed; no bits in a byte are transposed. @internal gui name="Transpose Bytes" */
}CRC_Transpose_Type;

/* CRCЭ ڿٳʼ */
typedef enum
{
    kCRC16_IBM,
    kCRC16_MAXIM,
    kCRC16_USB,
    kCRC16_MODBUS,
    kCRC16_CCITT,
    kCRC16_CCITTFalse,
    kCRC16_X25,
    kCRC16_XMODEM,
    kCRC16_DNP,
    kCRC32,
    kCRC32_MPEG2,
    kCRC_ProtocolNameMax,
}CRC_ProtocolType;

typedef struct
{
    uint8_t crcWidth;                   /*!< Selects 16 or 32-bit CRC protocol */
    uint32_t seed;                      /*!< Value of the seed (initial) CRC value */
    uint32_t polynomial;                /*!< Value of the polynomial for the CRC calculation */
    CRC_Transpose_Type writeTranspose;  /*!< Defines transpose configuration of the data written to the CRC data register */
    CRC_Transpose_Type readTranspose;   /*!< Defines transpose configuration of the value read from the CRC data register */
    bool complementRead;                /*!< Enables complement read of CRC data register */
} CRC_InitTypeDef;



//!< API functions
void CRC_QuickInit(CRC_ProtocolType type);
void CRC_Init(CRC_InitTypeDef * CRC_InitStruct);
//uint16_t CRC16_GenerateSoftware(const uint8_t *src, uint32_t len);
uint32_t CRC_Generate(uint8_t* data, uint32_t len);

#endif


