/* * Copyright (c) 2015, Freescale Semiconductor, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of Freescale Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
  
#ifndef __CH_LIB_DAC_H__
#define __CH_LIB_DAC_H__

#include <stdint.h>


#define HW_DAC0     (0x00)  //DAC0ģ
#define HW_DAC1     (0x01)  //DAC1ģ


//!< DAC ص
typedef void (*DAC_CallBackType)(void);


typedef enum
{
    kDAC_TriggerSoftware,  //
    kDAC_TriggerHardware,  //Ӳ
}DAC_TriggerSelect_Type;

/*DAC ͻģʽ ѡ */
typedef enum
{
    kDAC_Buffer_Disable,    //رDAC
    kDAC_Buffer_Normal,     //DAC
    kDAC_Buffer_Swing,      //dacҡģʽ
    kDAC_Buffer_OneTimeScan,//DACһ
}DAC_BufferMode_Type;

typedef enum
{
    kDAC_Reference_1,       /* οѹԴ VREF_OUT */
    kDAC_Reference_2,       /* οѹԴ VDDA */
}DAC_ReferenceSelect_Type;

typedef struct
{
    uint32_t                    instance;      //DACģ0~1
    DAC_TriggerSelect_Type      triggerMode;   //DACģʽ
    DAC_BufferMode_Type         bufferMode;    //
    DAC_ReferenceSelect_Type    referenceMode; //οԴѡ
}DAC_InitTypeDef;


//!< DACжϼDMAѡ
typedef enum
{
    kDAC_DMA_Disable,
    kDAC_IT_Disable,
    kDAC_IT_BufferPointer_WaterMark,
    kDAC_IT_BufferPointer_TopFlag,      /* when read pointer reach 0 */
    kDAC_IT_BufferPointer_BottomFlag,   /* when read pointer reach upper limit */
    kDAC_DMA_BufferPointer_WaterMark,
    kDAC_DMA_BufferPointer_TopFlag,    
    kDAC_DMA_BufferPointer_BottomFlag, 
}DAC_ITDMAConfig_Type;
 

typedef enum
{
    kDAC_WaterMark_1Word,
    kDAC_WaterMark_2Word,
    kDAC_WaterMark_3Word,
    kDAC_WaterMark_4Word,
}DAC_WaterMarkMode_Type;

 //!< API functions
void DAC_Init(DAC_InitTypeDef* DAC_InitStruct);
void DAC_CallbackInstall(uint8_t instance, DAC_CallBackType AppCBFun);
void DAC_ITDMAConfig(uint32_t instance, DAC_ITDMAConfig_Type config);
uint32_t DAC_GetBufferReadPointer(uint32_t instance);
void DAC_SetBufferReadPointer(uint32_t instance, uint32_t value);
void DAC_SetBufferUpperLimit(uint32_t instance, uint32_t value);
void DAC_ITDMAConfig(uint32_t instance, DAC_ITDMAConfig_Type config);
void DAC_SetBufferValue(uint32_t instance, uint16_t* buf, uint8_t len);
void DAC_SetWaterMark(uint32_t instance, DAC_WaterMarkMode_Type value);
void DAC_SoftwareStartConversion(uint32_t instance);

#endif



