/* * Copyright (c) 2015, Freescale Semiconductor, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of Freescale Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
#ifndef __CH_LIB_FLEXBUS_H__
#define __CH_LIB_FLEXBUS_H__

#include <stdint.h>
#include <stdbool.h>

//!< Flexbus ݶ䷽ʽѡ
#define kFLEXBUS_DataLeftAligned   (0x00)  //
#define kFLEXBUS_DataRightAligned  (0x01)  //Ҷ

//!< Flexbus ԶӦźʹ
#define kFLEXBUS_AutoAckEnable     (0x00)  //ԶӦʹ
#define kFLEXBUS_AutoAckDisable    (0x01)  //رԶӦ

//!< Flexbus ˿λѡ
#define kFLEXBUS_PortSize_8Bit     (0x01)  //˿ݿȣ8λ
#define kFLEXBUS_PortSize_16Bit    (0x02)  //˿ݿȣ16λ
#define kFLEXBUS_PortSize_32Bit    (0x00)  //˿ݿȣ32λ

//!< Flexbus Ƭѡźѡ 
#define kFLEXBUS_CS0               (0x00)  //0Ƭѡź
#define kFLEXBUS_CS1               (0x01)  //1Ƭѡź
#define kFLEXBUS_CS2               (0x02)  //2Ƭѡź
#define kFLEXBUS_CS3               (0x03)  //3Ƭѡź
#define kFLEXBUS_CS4               (0x04)  //4Ƭѡź
#define kFLEXBUS_CS5               (0x05)  //5Ƭѡź

//!< Flexbus ƬѡΧ οѡ
#define kFLEXBUS_ADSpace_64KByte        (0x00) //SRAMߴ 64k
#define kFLEXBUS_ADSpace_128KByte       (0x01) //SRAMߴ 128k
#define kFLEXBUS_ADSpace_512KByte       (0x07) //SRAMߴ 512k
#define kFLEXBUS_ADSpace_1MByte         (0x0F) //SRAMߴ 1m

//!< Flexbus BEźſģʽѡ
#define kFLEXBUS_BE_AssertedWrite       (0x00) //BEźŹܣд
#define kFLEXBUS_BE_AssertedReadWrite   (0x01) //BEźŹܣд


typedef enum
{
    kFLEXBUS_CSPMCR_Group1,
    kFLEXBUS_CSPMCR_Group2,  
    kFLEXBUS_CSPMCR_Group3,
    kFLEXBUS_CSPMCR_Group4,
    kFLEXBUS_CSPMCR_Group5,    
}FLEXBUS_PortMultiplexingSelect_Type;

//!< Flexus źŸѡ
#define kFLEXBUS_CSPMCR_GROUP1_ALE      (0x00)
#define kFLEXBUS_CSPMCR_GROUP1_CS1      (0x01)
#define kFLEXBUS_CSPMCR_GROUP1_TS       (0x02)
#define kFLEXBUS_CSPMCR_GROUP2_CS4      (0x00)
#define kFLEXBUS_CSPMCR_GROUP2_TSIZ0    (0x01)
#define kFLEXBUS_CSPMCR_GROUP2_BE_31_24 (0x02)
#define kFLEXBUS_CSPMCR_GROUP3_CS5      (0x00)
#define kFLEXBUS_CSPMCR_GROUP3_TSIZ1    (0x01)
#define kFLEXBUS_CSPMCR_GROUP3_BE_23_16 (0x02)
#define kFLEXBUS_CSPMCR_GROUP4_TBST     (0x00)
#define kFLEXBUS_CSPMCR_GROUP4_CS2      (0x01)
#define kFLEXBUS_CSPMCR_GROUP4_BE_15_8  (0x02)
#define kFLEXBUS_CSPMCR_GROUP5_TA       (0x00)
#define kFLEXBUS_CSPMCR_GROUP5_CS3      (0x01)
#define kFLEXBUS_CSPMCR_GROUP5_BE_7_0   (0x02)

//!< FLEXBUSʼṹ
typedef struct
{
    uint32_t div;             //ٶȷƵ
    uint32_t dataWidth;       //ݿ
    uint32_t baseAddress;     //豸ַ
    uint32_t ADSpaceMask;     //豸洢ռ
    uint32_t dataAlignMode;   //ݶ뷽ʽ
    uint32_t autoAckMode;     //ԶӦģʽ
    uint32_t ByteEnableMode;  //BEʹģʽ
    uint32_t CSn;             //Ƭѡźͨ
}FLEXBUS_InitTypeDef;

//!< ߼ѡ
typedef struct
{
    bool kFLEXBUS_brustWriteEnable;
    bool kFLEXBUS_brustReadEnable;
    bool kFLEXBUS_EXTS;
    uint32_t kFLEXBUS_ASET:2;
    uint32_t kFLEXBUS_RDAH:2;
    uint32_t kFLEXBUS_WRAH:2;
    uint32_t kFLEXBUS_WS:6;    
}FLEXBUS_AdvancedConfigTypeDef;

//!< API functions
void FLEXBUS_Init(FLEXBUS_InitTypeDef* FLEXBUS_InitStruct);
void FLEXBUS_PortMuxConfig(FLEXBUS_PortMultiplexingSelect_Type group, uint32_t config);
void FLEXBUS_AdvancedConfig(uint32_t CS, FLEXBUS_AdvancedConfigTypeDef* FLEXBUS_AdvancedConfigStruct);

#endif

