/* * Copyright (c) 2015, Freescale Semiconductor, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of Freescale Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 

#ifndef __CH_LIB_SPI_H__
#define __CH_LIB_SPI_H__

#include <stdint.h>
#include <stdbool.h>
  
//!< SPIģ豸
#define HW_SPI0     (0x00)
#define HW_SPI1     (0x01)
#define HW_SPI2     (0x02)

#define HW_CTAR0    (0x00)  
#define HW_CTAR1    (0x01)  

#define HW_SPI_CS0  (0x00)
#define HW_SPI_CS1  (0x01)
#define HW_SPI_CS2  (0x02)
#define HW_SPI_CS3  (0x03)

//!< SPI֡ʽѡ
typedef enum
{
    kSPI_CPOL0_CPHA0,
    kSPI_CPOL0_CPHA1,
    kSPI_CPOL1_CPHA0,
    kSPI_CPOL1_CPHA1
}SPI_FrameFormat_Type;

typedef enum
{
    kSPI_Master,
    kSPI_Slave,
} SPI_Mode_Type;

//!< interrupt and DMA select
typedef enum
{
    kSPI_IT_TCF,            //!< SPIһжʹ
    kSPI_DMA_TFFF,          //!< transmit FIFO full
    kSPI_DMA_RFDF,          //!< receive FIFO drain
}SPI_ITDMAConfig_Type;

/*!< ÿ֡λ */
typedef enum 
{
    kSPI_PCS_ReturnInactive  = 0,   //!< ɺCSźűδѡ״̬(ͨΪظߵƽ)
    kSPI_PCS_KeepAsserted  = 1,     //!< ɺCSźűƬѡ״̬(ͨΪ)
}SPI_PCS_Type;

typedef enum
{
    kSPI_MSB,      //!< ȷλ
    kSPI_LSB,      //!< ȷλ
} SPI_BitOlder_Type;

//!< ʼṹ
typedef struct
{
    uint32_t                instance;               //!< ģ
	SPI_Mode_Type           mode;                   //!< ģʽ
    uint8_t                 dataSize;               //!< ÿ֡жλ ͨΪ816
    SPI_BitOlder_Type       bitOrder;               //!< ȷλȷλ
    SPI_FrameFormat_Type    frameFormat;            //!< ֡ʽѡ
    uint32_t                baudrate;               //!< 
    uint32_t                ctar;                   //!< ֡ʽĴѡ 
}SPI_InitTypeDef;

//!< ٳʼṹ
    
#define SPI0_SCK_PA15_SOUT_PA16_SIN_PA17   (0x0000de80)
#define SPI0_SCK_PC05_SOUT_PC06_SIN_PC07   (0x0000ca90)
#define SPI0_SCK_PD01_SOUT_PD02_SIN_PD03   (0x0000c298)
#define SPI1_SCK_PD05_SOUT_PD06_SIN_PD07   (0x0000cbd9)
#define SPI1_SCK_PE02_SOUT_PE01_SIN_PE03   (0x0000c2a1)

#define SPI0_SOUT_PD02_SIN_PD03   (0x00008498)
//{ 0, 3, 2, 2, 2, 0}, //SPI0_SOUT_PD02_SIN_PD03 2

/*    
#define SPI0_SCK_PC05_SOUT_PC06_SIN_PC07   (0xca90U)
#define SPI0_SCK_PD01_SOUT_PD02_SIN_PD03   (0xc298U)
#define SPI1_SCK_PE02_SOUT_PE01_SIN_PE03   (0xc2a1U)
#define SPI0_SCK_PA15_SOUT_PA16_SIN_PA17   (0xde80U)
#define SPI2_SCK_PB21_SOUT_PB22_SIN_PB23   (0xea8aU)
#define SPI2_SCK_PD12_SOUT_PD13_SIN_PD14   (0xd89aU)
*/
//!< Callback Type
typedef void (*SPI_CallBackType)(void);

//!< API functions
void SPI_Init(SPI_InitTypeDef * SPI_InitStruct);
uint16_t SPI_ReadWriteByte(uint32_t instance,uint32_t ctar, uint16_t data, uint16_t CSn, SPI_PCS_Type csState);
void SPI_ITDMAConfig(uint32_t instance, SPI_ITDMAConfig_Type config, bool status);
void SPI_CallbackInstall(uint32_t instance, SPI_CallBackType AppCBFun);
uint32_t SPI_QuickInit(uint32_t MAP, SPI_FrameFormat_Type frameFormat, uint32_t baudrate);
void SPI_CTARConfig(uint32_t instance, uint32_t ctar, SPI_FrameFormat_Type frameFormat, uint8_t dataSize, uint8_t bitOrder, uint32_t baudrate);

/* FIFO functions */
void SPI_EnableTxFIFO(uint32_t instance, bool status);
void SPI_EnableRxFIFO(uint32_t instance, bool status);

#endif

