/* * Copyright (c) 2015, Freescale Semiconductor, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of Freescale Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
  
 #include "dac.h"
 #include "common.h"
 #if defined(DAC0)
 
#if (!defined(DAC_BASES))

#ifdef DAC0
#define DAC_BASES   {DAC0}
#elif  DAC1
#define DAC_BASES   {DAC0, DAC1}
#endif

#endif
 
 /* global vars */
static DAC_Type * const DAC_InstanceTable[] = DAC_BASES;
static DAC_CallBackType DAC_CallBackTable[ARRAY_SIZE(DAC_InstanceTable)] = {NULL};
static const IRQn_Type DAC_IRQnTable[] = 
{
    DAC0_IRQn,
#ifdef DAC1
    DAC1_IRQn,
#endif
};

/**
 * @brief  ʼDACģ
 * @code
 *  DAC_InitTypeDef DAC_InitStruct = {0};
 *  DAC_InitStruct.bufferMode = kDAC_Buffer_Swing; 
 *  DAC_InitStruct.instance = HW_DAC0;
 *  DAC_InitStruct.referenceMode = kDAC_Reference_2; 
 *  DAC_InitStruct.triggerMode = kDAC_TriggerSoftware; 
 *  DAC_Init(&DAC_InitStruct);
 * @endcode
 * @param  DAC_InitTypeDef: 
 *         instance      :ģ
 *         @arg HW_DAC0 :оƬDAC0 ģ
 *         @arg HW_DAC1 :оƬDAC1 ģ
 * @retval None
 */
void DAC_Init(DAC_InitTypeDef* DAC_InitStruct)
{
    SIM->SCGC6 |= SIM_SCGC6_DAC0_MASK;
    /* trigger select */
    switch(DAC_InitStruct->triggerMode)
    {
        case kDAC_TriggerSoftware:
            DAC_InstanceTable[DAC_InitStruct->instance]->C0 |= DAC_C0_DACTRGSEL_MASK;
            break;
        case kDAC_TriggerHardware:
            DAC_InstanceTable[DAC_InitStruct->instance]->C0 &= ~DAC_C0_DACTRGSEL_MASK;
            break;
        default:
            break;
    }
    /* buffer mode */
    switch(DAC_InitStruct->bufferMode)
    {
        case kDAC_Buffer_Disable:
            DAC_InstanceTable[DAC_InitStruct->instance]->C1 &= ~DAC_C1_DACBFEN_MASK;
            break;
        case kDAC_Buffer_Normal:
            DAC_InstanceTable[DAC_InitStruct->instance]->C1 |= DAC_C1_DACBFEN_MASK;
            DAC_InstanceTable[DAC_InitStruct->instance]->C1 &= ~DAC_C1_DACBFMD_MASK;
            DAC_InstanceTable[DAC_InitStruct->instance]->C1 |= DAC_C1_DACBFMD(0);
            break;
        case kDAC_Buffer_Swing:
            DAC_InstanceTable[DAC_InitStruct->instance]->C1 |= DAC_C1_DACBFEN_MASK;
            DAC_InstanceTable[DAC_InitStruct->instance]->C1 &= ~DAC_C1_DACBFMD_MASK;
            DAC_InstanceTable[DAC_InitStruct->instance]->C1 |= DAC_C1_DACBFMD(1);
            break;
        case kDAC_Buffer_OneTimeScan:
            DAC_InstanceTable[DAC_InitStruct->instance]->C1 |= DAC_C1_DACBFEN_MASK;
            DAC_InstanceTable[DAC_InitStruct->instance]->C1 &= ~DAC_C1_DACBFMD_MASK;
            DAC_InstanceTable[DAC_InitStruct->instance]->C1 |= DAC_C1_DACBFMD(2);
            break;
        default:
            break;
    }
    /* reference */
    switch(DAC_InitStruct->referenceMode)
    {
        case kDAC_Reference_1:
            DAC_InstanceTable[DAC_InitStruct->instance]->C0 &= ~DAC_C0_DACRFS_MASK;
            break;
        case kDAC_Reference_2:
            DAC_InstanceTable[DAC_InitStruct->instance]->C0 |= DAC_C0_DACRFS_MASK;
            break;
        default:
            break;
    }
    /* clear flags */
    DAC_InstanceTable[DAC_InitStruct->instance]->SR &= ~DAC_SR_DACBFRPBF_MASK;
    DAC_InstanceTable[DAC_InitStruct->instance]->SR &= ~DAC_SR_DACBFRPTF_MASK;
    DAC_InstanceTable[DAC_InitStruct->instance]->SR &= ~DAC_SR_DACBFWMF_MASK;
    /* enable module */
    DAC_InstanceTable[DAC_InitStruct->instance]->C0 |= DAC_C0_DACEN_MASK;
}
 
/**
 * @brief  DACģbufferָ
 *
 * @param  instance      :ģ
 *         @arg HW_DAC0 :оƬDAC0 ģ
 *         @arg HW_DAC1 :оƬDAC1 ģ
 * @note   DACǰת buffer ָ λ
 * @retval None
 */
uint32_t DAC_GetBufferReadPointer(uint32_t instance)
{
    return (DAC_InstanceTable[instance]->C2 & DAC_C2_DACBFRP_MASK) >> DAC_C2_DACBFRP_SHIFT;
}

/**
 * @brief  DACģbufferָ
 *
 * @param  instance     :ģ
 *         @arg HW_DAC0 :оƬDAC0 ģ
 *         @arg HW_DAC1 :оƬDAC1 ģ
 * @param  value        :ָλ 0-15
 * @retval None
 */
void DAC_SetBufferReadPointer(uint32_t instance, uint32_t value)
{
    DAC_InstanceTable[instance]->C2 &= ~DAC_C2_DACBFRP_MASK;
    DAC_InstanceTable[instance]->C2 |= DAC_C2_DACBFRP(value);
}

/**
 * @brief  DACģȡbufferָʱֵ
 *
 * @param  instance     :ģ
 *         @arg HW_DAC0 :оƬDAC0 ģ
 *         @arg HW_DAC1 :оƬDAC1 ģ
 * @param  value        :ָλ 0-15
 * @retval None
 */
void DAC_SetBufferUpperLimit(uint32_t instance, uint32_t value)
{
    DAC_InstanceTable[instance]->C2 &= ~DAC_C2_DACBFUP_MASK;
    DAC_InstanceTable[instance]->C2 |= DAC_C2_DACBFUP(value);
}

/**
 * @brief  DACģжϺDMA
 *
 * @param  instance     :ģ
 *         @arg HW_DAC0 :оƬDAC0 ģ
 *         @arg HW_DAC1 :оƬDAC1 ģ
 * @param  config       :ѡ
 *         @arg kDAC_DMA_Disable                : ֹDAC DMA
 *         @arg kDAC_IT_Disable                 : ֹDAC жϹ
 *         @arg kDAC_IT_BufferPointer_WaterMark : DAC ˮλж
 *         @arg kDAC_IT_BufferPointer_TopFlag   : DAC ReadPointer = 0ж
 *         @arg kDAC_IT_BufferPointer_BottomFlag: ʼDAC ReadPointer = UpLimit ж
 *         @arg kDAC_DMA_BufferPointer_WaterMark: 
 *         @arg kDAC_DMA_BufferPointer_TopFlag  :
 *         @arg kDAC_DMA_BufferPointer_BottomFlag:
 * @retval None
 */
void DAC_ITDMAConfig(uint32_t instance, DAC_ITDMAConfig_Type config)
{
    switch(config)
    {
        case kDAC_IT_Disable:
            NVIC_DisableIRQ(DAC_IRQnTable[instance]);
            break;
        case kDAC_DMA_Disable:
            DAC_InstanceTable[instance]->C1 &= ~DAC_C1_DMAEN_MASK;
            break;
        case kDAC_IT_BufferPointer_WaterMark:
            DAC_InstanceTable[instance]->C0 |= DAC_C0_DACBWIEN_MASK;
            NVIC_EnableIRQ(DAC_IRQnTable[instance]);
            break;
        case kDAC_IT_BufferPointer_TopFlag:
            DAC_InstanceTable[instance]->C0 |= DAC_C0_DACBTIEN_MASK;
            NVIC_EnableIRQ(DAC_IRQnTable[instance]);
            break;
        case kDAC_IT_BufferPointer_BottomFlag:
            DAC_InstanceTable[instance]->C0 |= DAC_C0_DACBBIEN_MASK;
            NVIC_EnableIRQ(DAC_IRQnTable[instance]);
            break;
        case kDAC_DMA_BufferPointer_WaterMark:
            DAC_InstanceTable[instance]->C0 |= DAC_C0_DACBWIEN_MASK;
            DAC_InstanceTable[instance]->C1 |= DAC_C1_DMAEN_MASK;
            break;
        case kDAC_DMA_BufferPointer_TopFlag:
            DAC_InstanceTable[instance]->C0 |= DAC_C0_DACBTIEN_MASK;
            DAC_InstanceTable[instance]->C1 |= DAC_C1_DMAEN_MASK;
            break;
        case kDAC_DMA_BufferPointer_BottomFlag:
            DAC_InstanceTable[instance]->C0 |= DAC_C0_DACBBIEN_MASK;
            DAC_InstanceTable[instance]->C1 |= DAC_C1_DMAEN_MASK;
            break;
        default:
            break;
    }
}

/**
 * @brief  DACʼ
 *
 * @param  instance     :ģ
 *         @arg HW_DAC0 :оƬDAC0 ģ
 *         @arg HW_DAC1 :оƬDAC1 ģ
 * @note   will only trigger once and will advence the read pointer by one step
 * @retval None
 */
void DAC_SoftwareStartConversion(uint32_t instance)
{
    DAC_InstanceTable[instance]->C0 |= DAC_C0_DACSWTRG_MASK;
}

/**
 * @brief  DAC Bufferˮλ
 *
 * @param  instance     :ģ
 *         @arg HW_DAC0 :оƬDAC0 ģ
 *         @arg HW_DAC1 :оƬDAC1 ģ
 * @param  value        :ˮλֵ 0-15
 * @retval None
 */
void DAC_SetWaterMark(uint32_t instance, DAC_WaterMarkMode_Type value)
{
    DAC_InstanceTable[instance]->C1 &= ~DAC_C1_DACBFWM_MASK;
	switch(value)
	{
		case kDAC_WaterMark_1Word:
			DAC_InstanceTable[instance]->C1 |= DAC_C1_DACBFWM(0);
			break;
		case kDAC_WaterMark_2Word:
			DAC_InstanceTable[instance]->C1 |= DAC_C1_DACBFWM(1);
			break;
		case kDAC_WaterMark_3Word:
			DAC_InstanceTable[instance]->C1 |= DAC_C1_DACBFWM(2);
			break;
		case kDAC_WaterMark_4Word:
			DAC_InstanceTable[instance]->C1 |= DAC_C1_DACBFWM(3);
			break;
		default:break;
	}
}

/**
 * @brief  DAC 
 *
 * @param  instance     :ģ
 *         @arg HW_DAC0 :оƬDAC0 ģ
 *         @arg HW_DAC1 :оƬDAC1 ģ
 * @param  buf          :ָ
 * @param  len          :
 * @retval None
 */
void DAC_SetBufferValue(uint32_t instance, uint16_t* buf, uint8_t len)
{
	uint8_t i;
	for(i = 0; i < len; i++)
	{
        DAC_InstanceTable[instance]->DAT[i].DATL = (buf[i] & 0x00FF);
        DAC_InstanceTable[instance]->DAT[i].DATH = (buf[i] & 0xFF00)>>8;
	}                       
}

/**
 * @brief  עжϻص
 * @param  instance: DAC ģ
 *         @arg HW_DAC0  :DAC0ģ
 *         @arg HW_DAC1  :DAC1ģ
 * @param AppCBFun: صָ
 * @retval None
 * @note ڴ˺ľӦӦʵ
 */
void DAC_CallbackInstall(uint8_t instance, DAC_CallBackType AppCBFun)
{
    if(AppCBFun != NULL)
    {
        DAC_CallBackTable[instance] = AppCBFun;
    }
}


void DAC0_IRQHandler(void)
{
    /* clear IT pending bit */
    uint32_t C0 = DAC_InstanceTable[HW_DAC0]->C0;
    if(C0 & DAC_C0_DACBWIEN_MASK)
    {
        DAC_InstanceTable[HW_DAC0]->SR &= ~DAC_SR_DACBFWMF_MASK;
    }
    if(C0 & DAC_C0_DACBBIEN_MASK)
    {
        DAC_InstanceTable[HW_DAC0]->SR &= ~DAC_SR_DACBFRPBF_MASK;
    }
    if(C0 & DAC_C0_DACBTIEN_MASK)
    {
        DAC_InstanceTable[HW_DAC0]->SR &= ~DAC_SR_DACBFRPTF_MASK;
    }
    /* call back */
    if(DAC_CallBackTable[HW_DAC0] != NULL)
    {
        DAC_CallBackTable[HW_DAC0]();
    }  
}


void DAC1_IRQHandler(void)
{
    /* clear IT pending bit */
    uint32_t C0 = DAC_InstanceTable[HW_DAC0]->C0;
    if(C0 & DAC_C0_DACBWIEN_MASK)
    {
        DAC_InstanceTable[HW_DAC0]->SR &= ~DAC_SR_DACBFWMF_MASK;
    }
    if(C0 & DAC_C0_DACBBIEN_MASK)
    {
        DAC_InstanceTable[HW_DAC0]->SR &= ~DAC_SR_DACBFRPBF_MASK;
    }
    if(C0 & DAC_C0_DACBTIEN_MASK)
    {
        DAC_InstanceTable[HW_DAC0]->SR &= ~DAC_SR_DACBFRPTF_MASK;
    }
    /* call back */
    if(DAC_CallBackTable[HW_DAC1] != NULL)
    {
        DAC_CallBackTable[HW_DAC1]();
    }  
}
 
#endif
