/* * Copyright (c) 2015, Freescale Semiconductor, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of Freescale Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 

#include "gpio.h"

/* leagacy support for Kineis Z version(inital version) */
#if (!defined(GPIO_BASES))

#ifdef PTF
#define GPIO_BASES {PTA, PTB, PTC, PTD, PTE, PTF}
#define PORT_BASES {PORTA, PORTB, PORTC, PORTD, PORTE, PORTF}
#else
#define GPIO_BASES {PTA, PTB, PTC, PTD, PTE}
#define PORT_BASES {PORTA, PORTB, PORTC, PORTD, PORTE}
#endif


#endif

/* gloabl vars */
static GPIO_Type * const GPIO_InstanceTable[] = GPIO_BASES;
static PORT_Type * const PORT_InstanceTable[] = PORT_BASES;
static GPIO_CallBackType GPIO_CallBackTable[ARRAY_SIZE(PORT_InstanceTable)] = {NULL};
static const uint32_t SIM_GPIOClockGateTable[] =
{
    SIM_SCGC5_PORTA_MASK,
    SIM_SCGC5_PORTB_MASK,
    SIM_SCGC5_PORTC_MASK,
    SIM_SCGC5_PORTD_MASK,
    SIM_SCGC5_PORTE_MASK,
};
static const IRQn_Type GPIO_IRQnTable[] = 
{
    PORTA_IRQn,
    PORTB_IRQn,
    PORTC_IRQn,
    PORTD_IRQn,
    PORTE_IRQn,
};

//! @defgroup CHKinetis
//! @{


//! @defgroup GPIO
//! @brief GPIO API functions
//! @{

 /**
 * @brief Ÿù ᱻܶģ
 * @note  ùܿɲο Reference Manual  Signal Multiplexing and Signal Descriptions ½ 
 * @code
 *      // һPORTA˿ڵ3Ÿó1ģʽ.
 *      PORT_PinMuxConfig(HW_GPIOA, 3, kPinAlt1);
 * @endcode
 * @param  instance: GPIOģ 
 *         @arg HW_GPIOA :оƬPORTA˿
 *         @arg HW_GPIOB :оƬPORTB˿
 *         @arg HW_GPIOC :оƬPORTC˿
 *         @arg HW_GPIOD :оƬPORTD˿
 *         @arg HW_GPIOE :оƬPORTE˿
 * @param  pin  :˿ϵź 0~31
 * @param  pinMux    :ùѡͬĸֵͬĹ
 *         @arg kPinAlt0 :Ÿó0ģʽ
 *         @arg        . : .
 *         @arg        . : .
 *         @arg        . : .
 *         @arg kPinAlt7 :Ÿó7ģʽ
 * @retval None
 */
void PORT_PinMuxConfig(uint32_t instance, uint8_t pin, PORT_PinMux_Type pinMux)
{
    SIM->SCGC5 |= SIM_GPIOClockGateTable[instance];
    PORT_InstanceTable[instance]->PCR[pin] &= ~(PORT_PCR_MUX_MASK);
    PORT_InstanceTable[instance]->PCR[pin] |=  PORT_PCR_MUX(pinMux);
}
 /**
 * @brief  һŵ蹦 ûһ㲻ص
 * @code
 *      // PORTA˿ڵ3Ϊ()
 *      PORT_PinPullConfig(HW_GPIOA, 3, kPullUp);
 * @endcode
 * @param  instance: GPIOģ
 *         @arg HW_GPIOA :оƬPORTA˿
 *         @arg HW_GPIOB :оƬPORTB˿
 *         @arg HW_GPIOC :оƬPORTC˿
 *         @arg HW_GPIOD :оƬPORTD˿
 *         @arg HW_GPIOE :оƬPORTE˿
 * @param  pin  :˿ϵź 0~31
 * @param  pull      :ŵģʽѡ
 *         @arg kPullDisabled :رյ
 *         @arg kPullUp       :
 *         @arg kPullDown     :
 * @retval None
 */
void PORT_PinPullConfig(uint32_t instance, uint8_t pin, PORT_Pull_Type pull)
{
    SIM->SCGC5 |= SIM_GPIOClockGateTable[instance];
    switch(pull)
    {
        case kPullDisabled:
            PORT_InstanceTable[instance]->PCR[pin] &= ~PORT_PCR_PE_MASK;
            break;
        case kPullUp:
            PORT_InstanceTable[instance]->PCR[pin] |= PORT_PCR_PE_MASK;
            PORT_InstanceTable[instance]->PCR[pin] |= PORT_PCR_PS_MASK;
            break;
        case kPullDown:
            PORT_InstanceTable[instance]->PCR[pin] |= PORT_PCR_PE_MASK;
            PORT_InstanceTable[instance]->PCR[pin] &= ~PORT_PCR_PS_MASK;
            break;
        default:
            break;
    }
}
/**
 * @brief  ˿ŵĿ©״̬ ûһ㲻ص
 * @code
 *      // PORTA˿ڵ3Ϊ©״̬ 
 *      PORT_PinOpenDrainConfig(HW_GPIOA, 3, ENABLE);
 * @endcode
 * @param  instance: GPIOģ
 *         @arg HW_GPIOA :оƬPORTA˿
 *         @arg HW_GPIOB :оƬPORTB˿
 *         @arg HW_GPIOC :оƬPORTC˿
 *         @arg HW_GPIOD :оƬPORTD˿
 *         @arg HW_GPIOE :оƬPORTE˿
 * @param  pin  :˿ϵź 0~31
 * @param  newState  :ܿؿ
 *         @arg ENABLE   :
 *         @arg DISABLE  :رչ
 * @retval None
 */
void PORT_PinOpenDrainConfig(uint32_t instance, uint8_t pin, FunctionalState newState)
{
    SIM->SCGC5 |= SIM_GPIOClockGateTable[instance];
    (newState == ENABLE) ? (PORT_InstanceTable[instance]->PCR[pin] |= PORT_PCR_ODE_MASK):(PORT_InstanceTable[instance]->PCR[pin] &= ~PORT_PCR_ODE_MASK);
}

/**
 * @brief  ˿ŵĿԴ˲ ΪʱЧ
 * @code
 *      // PORTA˿ڵ3Ϊ©״̬ 
 *      PORT_PinPassiveFilterConfig(HW_GPIOA, 3, ENABLE);
 * @endcode
 * @param  instance: GPIOģ
 *         @arg HW_GPIOA :оƬPORTA˿
 *         @arg HW_GPIOB :оƬPORTB˿
 *         @arg HW_GPIOC :оƬPORTC˿
 *         @arg HW_GPIOD :оƬPORTD˿
 *         @arg HW_GPIOE :оƬPORTE˿
 * @param  pin  :˿ϵź 0~31
 * @param  newState  :ܿؿ
 *         @arg ENABLE   :
 *         @arg DISABLE  :رչ
 * @retval None
 */
void PORT_PinPassiveFilterConfig(uint32_t instance, uint8_t pin, FunctionalState newState)
{
    SIM->SCGC5 |= SIM_GPIOClockGateTable[instance];
    (newState == ENABLE) ? (PORT_InstanceTable[instance]->PCR[pin] |= PORT_PCR_PFE_MASK):(PORT_InstanceTable[instance]->PCR[pin] &= ~PORT_PCR_PFE_MASK);
}

 /**
 * @brief  Ϊ뻹  ûһ㲻ص
 * @note   ֻеΪGPIOʱ
 * @code
 *      // PORTB˿ڵ3
 *      GPIO_PinConfig(HW_GPIOB, 3, kInpput);
 * @endcode
 * @param  instance: GPIOģ
 *         @arg HW_GPIOA :оƬPORTA˿
 *         @arg HW_GPIOB :оƬPORTB˿
 *         @arg HW_GPIOC :оƬPORTC˿
 *         @arg HW_GPIOD :оƬPORTD˿
 *         @arg HW_GPIOE :оƬPORTE˿
 * @param  pin  :˿ϵź 0~31
 * @param  mode      :
 *         @arg kInpput  :빦ѡ
 *         @arg kOutput  :ѡ
 * @retval None
 */
void GPIO_PinConfig(uint32_t instance, uint8_t pin, GPIO_PinConfig_Type mode)
{
    SIM->SCGC5 |= SIM_GPIOClockGateTable[instance];
    (mode == kOutput) ? (GPIO_InstanceTable[instance]->PDDR |= (1 << pin)):(GPIO_InstanceTable[instance]->PDDR &= ~(1 << pin));
}

 /**
 * @brief  GPIOʼ
 * @code
 *    //ʼPORTB˿ڵ10Ϊ
 *    GPIO_InitTypeDef GPIO_InitStruct1;      //һṹ
 *    GPIO_InitStruct1.instance = HW_GPIOB;   //ѡPORTB˿
 *    GPIO_InitStruct1.mode = kGPIO_Mode_OPP; //
 *    GPIO_InitStruct1.pinx = 10;             //ѡ10
 *    //óʼGPIO 
 *    GPIO_Init(&GPIO_InitStruct1);
 * @endcode
 * @param  GPIO_InitStruct: GPIOʼṹ壬״̬  
           GPIO_InitStruct.instance   :˿ں HW_GPIOA ~ HW_GPIOE
 * @retval None
 */
void GPIO_Init(GPIO_InitTypeDef * GPIO_InitStruct)
{
    /* config state */
    switch(GPIO_InitStruct->mode)
    {
        case kGPIO_Mode_IFT:
            PORT_PinPullConfig(GPIO_InitStruct->instance, GPIO_InitStruct->pinx, kPullDisabled);
            PORT_PinOpenDrainConfig(GPIO_InitStruct->instance, GPIO_InitStruct->pinx, DISABLE);
            GPIO_PinConfig(GPIO_InitStruct->instance, GPIO_InitStruct->pinx, kInput);
            break;
        case kGPIO_Mode_IPD:
            PORT_PinPullConfig(GPIO_InitStruct->instance, GPIO_InitStruct->pinx, kPullDown);
            PORT_PinOpenDrainConfig(GPIO_InitStruct->instance, GPIO_InitStruct->pinx, DISABLE);
            GPIO_PinConfig(GPIO_InitStruct->instance, GPIO_InitStruct->pinx, kInput);
            break;
        case kGPIO_Mode_IPU:
            PORT_PinPullConfig(GPIO_InitStruct->instance, GPIO_InitStruct->pinx, kPullUp);
            PORT_PinOpenDrainConfig(GPIO_InitStruct->instance, GPIO_InitStruct->pinx, DISABLE);
            GPIO_PinConfig(GPIO_InitStruct->instance, GPIO_InitStruct->pinx, kInput);
            break;
        case kGPIO_Mode_OOD:
            PORT_PinPullConfig(GPIO_InitStruct->instance, GPIO_InitStruct->pinx, kPullUp);
            PORT_PinOpenDrainConfig(GPIO_InitStruct->instance, GPIO_InitStruct->pinx, ENABLE);
            GPIO_PinConfig(GPIO_InitStruct->instance, GPIO_InitStruct->pinx, kOutput);
            break;
        case kGPIO_Mode_OPP:
            PORT_PinPullConfig(GPIO_InitStruct->instance, GPIO_InitStruct->pinx, kPullDisabled);
            PORT_PinOpenDrainConfig(GPIO_InitStruct->instance, GPIO_InitStruct->pinx, DISABLE);
            GPIO_PinConfig(GPIO_InitStruct->instance, GPIO_InitStruct->pinx, kOutput);
            break;
        default:
            break;					
    }
    /* config pinMux */
    PORT_PinMuxConfig(GPIO_InitStruct->instance, GPIO_InitStruct->pinx, kPinAlt1);
}

 /**
 * @brief  ٳʼһGPIO ʵGPIO_Init
 * @code
 *      //ʼPORTB˿ڵ10Ϊ
 *      GPIO_QuickInit(HW_GPIOB, 10, kGPIO_Mode_OPP);
 * @endcode
 * @param  instance: GPIOģ
 *         @arg HW_GPIOA :оƬPORTA˿
 *         @arg HW_GPIOB :оƬPORTB˿
 *         @arg HW_GPIOC :оƬPORTC˿
 *         @arg HW_GPIOD :оƬPORTD˿
 *         @arg HW_GPIOE :оƬPORTE˿
 * @param  pin  :˿ϵź 0~31
 * @param  mode  :Źģʽ
 *         @arg kGPIO_Mode_IFT :
 *         @arg kGPIO_Mode_IPD :
 *         @arg kGPIO_Mode_IPU :
 *         @arg kGPIO_Mode_OOD :©
 *         @arg kGPIO_Mode_OPP :
 * @retval None
 */
uint8_t GPIO_QuickInit(uint32_t instance, uint32_t pinx, GPIO_Mode_Type mode)
{
    GPIO_InitTypeDef GPIO_InitStruct1;
    GPIO_InitStruct1.instance = instance;
    GPIO_InitStruct1.mode = mode;
    GPIO_InitStruct1.pinx = pinx;
    GPIO_Init(&GPIO_InitStruct1);
    return  instance;
}

 /**
 * @brief  ָߵƽߵ͵ƽ
 * @note   ó
 * @code
 *      //PORTB˿ڵ10ߵƽ
 *      GPIO_WriteBit(HW_GPIOB, 10, 1);
 * @endcode
 * @param  instance: GPIOģ
 *         @arg HW_GPIOA :оƬPORTA˿
 *         @arg HW_GPIOB :оƬPORTB˿
 *         @arg HW_GPIOC :оƬPORTC˿
 *         @arg HW_GPIOD :оƬPORTD˿
 *         @arg HW_GPIOE :оƬPORTE˿
 * @param  pin  :˿ϵź 0~31
 * @param  data   : ŵĵƽ״̬  
 *         @arg 0 : ͵ƽ 
 *         @arg 1 : ߵƽ
 * @retval None
 */
void GPIO_WriteBit(uint32_t instance, uint8_t pin, uint8_t data)
{
    SIM->SCGC5 |= SIM_GPIOClockGateTable[instance];
    (data) ? (GPIO_InstanceTable[instance]->PSOR |= (1 << pin)):(GPIO_InstanceTable[instance]->PCOR |= (1 << pin));
}

void GPIO_SetBit(uint32_t instance, uint32_t pin)
{
    GPIO_InstanceTable[instance]->PSOR |= (1 << pin);
}

void GPIO_ResetBit(uint32_t instance, uint32_t pin)
{
    GPIO_InstanceTable[instance]->PCOR |= (1 << pin);
}

 /**
 * @brief  ȡһϵĵƽ״̬
 * @code
 *      //ȡPORTB˿ڵ10ŵĵƽ״̬
 *      uint8_t status ; //ڴ洢ŵ״̬
 *      status = GPIO_ReadBit(HW_GPIOB, 10); //ȡŵ״̬洢status
 * @endcode
 * @param  instance: GPIOģ
 *         @arg HW_GPIOA :оƬPORTA˿
 *         @arg HW_GPIOB :оƬPORTB˿
 *         @arg HW_GPIOC :оƬPORTC˿
 *         @arg HW_GPIOD :оƬPORTD˿
 *         @arg HW_GPIOE :оƬPORTE˿
 * @param  pin  :˿ϵź 0~31
 * @retval 
 *         @arg 0 : ͵ƽ
 *         @arg 1 : ߵƽ
 */
uint8_t GPIO_ReadBit(uint32_t instance, uint8_t pin)
{
    /* input or output */
    if(((GPIO_InstanceTable[instance]->PDDR) >> pin) & 0x01)
    {
        return ((GPIO_InstanceTable[instance]->PDOR >> pin) & 0x01);
    }
    else
    {
        return ((GPIO_InstanceTable[instance]->PDIR >> pin) & 0x01);
    }
}

 /**
 * @brief  תһŵĵƽ״̬
 * @code
 *      //תPORTB˿ڵ10ŵĵƽ״̬
 *      GPIO_ToggleBit(HW_GPIOB, 10); 
 * @endcode
 * @param  instance: GPIOģ
 *         @arg HW_GPIOA :оƬPORTA˿
 *         @arg HW_GPIOB :оƬPORTB˿
 *         @arg HW_GPIOC :оƬPORTC˿
 *         @arg HW_GPIOD :оƬPORTD˿
 *         @arg HW_GPIOE :оƬPORTE˿
 * @param  pin  :˿ϵź 0~31
 * @retval None
 */
void GPIO_ToggleBit(uint32_t instance, uint8_t pin)
{
    GPIO_InstanceTable[instance]->PTOR |= (1 << pin);
}

/**
 * @brief  ȡһ˿32λ
 * @code
 *      //ȡPORTB˿ڵŵĵƽ״̬
 *      uint32_t status ; //ڴ洢ŵ״̬
 *      status = GPIO_ReadPort(HW_GPIOB); //ȡŵ״̬洢status
 * @endcode
 * @param  instance: GPIOģ
 *         @arg HW_GPIOA :оƬPORTA˿
 *         @arg HW_GPIOB :оƬPORTB˿
 *         @arg HW_GPIOC :оƬPORTC˿
 *         @arg HW_GPIOD :оƬPORTD˿
 *         @arg HW_GPIOE :оƬPORTE˿
 * @retval ˿ڵ32λ
 */
uint32_t GPIO_ReadPort(uint32_t instance)
{
    return (GPIO_InstanceTable[instance]->PDIR);
}
/**
 * @brief  һ˿д32λ
 * @code
 *      //PORTB˿д0xFFFFFFFF
 *      GPIO_WriteByte(HW_GPIOB, 0xFFFFFFFF); 
 * @endcode
 * @param  instance: GPIOģ
 *         @arg HW_GPIOA :оƬPORTA˿
 *         @arg HW_GPIOB :оƬPORTB˿
 *         @arg HW_GPIOC :оƬPORTC˿
 *         @arg HW_GPIOD :оƬPORTD˿
 *         @arg HW_GPIOE :оƬPORTE˿
 * @param  data  :32λ
 * @retval None
 */
void GPIO_WritePort(uint32_t instance, uint32_t data)
{
    GPIO_InstanceTable[instance]->PDOR = data;
}


/**
 * @brief  GPIOжͻDMA
 * @code
 *      //PORTB˿ڵ10Ϊ½شж
 *      GPIO_ITDMAConfig(HW_GPIOB, 10, kGPIO_IT_FallingEdge, true); 
 * @endcode
 * @param  instance: GPIOģ
 *         @arg HW_GPIOA :оƬPORTA˿
 *         @arg HW_GPIOB :оƬPORTB˿
 *         @arg HW_GPIOC :оƬPORTC˿
 *         @arg HW_GPIOD :оƬPORTD˿
 *         @arg HW_GPIOE :оƬPORTE˿
 * @param  pin  :˿ϵź 0~31
 * @param config: ģʽ
 *         @arg kGPIO_DMA_RisingEdge DMAش
 *         @arg kGPIO_DMA_FallingEdge DMA½ش
 *         @arg kGPIO_DMA_RisingFallingEdge DMA½ض
 *         @arg kGPIO_IT_Low ͵ƽж
 *         @arg kGPIO_IT_RisingEdge شж
 *         @arg kGPIO_IT_FallingEdge ½شж
 *         @arg kGPIO_IT_RisingFallingEdge ½ضж
 *         @arg kGPIO_IT_High ߵƽж
 * @retval None
 */
void GPIO_ITDMAConfig(uint32_t instance, uint8_t pin, GPIO_ITDMAConfig_Type config, bool status)
{
    /* init moudle */
    SIM->SCGC5 |= SIM_GPIOClockGateTable[instance];
    PORT_InstanceTable[instance]->PCR[pin] &= ~PORT_PCR_IRQC_MASK;
    
    if(!status)
    {
        NVIC_DisableIRQ(GPIO_IRQnTable[instance]);
        return;
    }
    
    switch(config)
    {
        case kGPIO_DMA_RisingEdge:
            PORT_InstanceTable[instance]->PCR[pin] |= PORT_PCR_IRQC(1);
            break;
        case kGPIO_DMA_FallingEdge:
            PORT_InstanceTable[instance]->PCR[pin] |= PORT_PCR_IRQC(2);
            break;
        case kGPIO_DMA_RisingFallingEdge:
            PORT_InstanceTable[instance]->PCR[pin] |= PORT_PCR_IRQC(3);
            break;
        case kGPIO_IT_Low:
            PORT_InstanceTable[instance]->PCR[pin] |= PORT_PCR_IRQC(8);
            NVIC_EnableIRQ(GPIO_IRQnTable[instance]);
            break;
        case kGPIO_IT_RisingEdge:
            PORT_InstanceTable[instance]->PCR[pin] |= PORT_PCR_IRQC(9);
            NVIC_EnableIRQ(GPIO_IRQnTable[instance]);
            break;
        case kGPIO_IT_FallingEdge:
            PORT_InstanceTable[instance]->PCR[pin] |= PORT_PCR_IRQC(10);
            NVIC_EnableIRQ(GPIO_IRQnTable[instance]);
            break;
        case kGPIO_IT_RisingFallingEdge:
            PORT_InstanceTable[instance]->PCR[pin] |= PORT_PCR_IRQC(11);
            NVIC_EnableIRQ(GPIO_IRQnTable[instance]);
            break;
        case kGPIO_IT_High:
            PORT_InstanceTable[instance]->PCR[pin] |= PORT_PCR_IRQC(12);
            NVIC_EnableIRQ(GPIO_IRQnTable[instance]);
            break;
        default:
            break;
    }
}
/**
 * @brief  עжϻص
 * @param  instance: GPIOģжں
 *         @arg HW_GPIOA :оƬPORTA˿ж
 *         @arg HW_GPIOB :оƬPORTB˿ж
 *         @arg HW_GPIOC :оƬPORTC˿ж
 *         @arg HW_GPIOD :оƬPORTD˿ж
 *         @arg HW_GPIOE :оƬPORTE˿ж
 * @param AppCBFun: صָ
 * @retval None
 * @note ڴ˺ľӦӦʵ
 */
void GPIO_CallbackInstall(uint32_t instance, GPIO_CallBackType AppCBFun)
{
    /* init moudle */
    SIM->SCGC5 |= SIM_GPIOClockGateTable[instance];
    if(AppCBFun != NULL)
    {
        GPIO_CallBackTable[instance] = AppCBFun;
    }
}

//! @}


/**
 * @brief  жϴ
 * @param  PORTA_IRQHandler :оƬPORTA˿жϺ
 *         PORTB_IRQHandler :оƬPORTB˿жϺ
 *         PORTC_IRQHandler :оƬPORTC˿жϺ
 *         PORTD_IRQHandler :оƬPORTD˿жϺ
 *         PORTE_IRQHandler :оƬPORTE˿жϺ
 * @note ڲж¼
 */
void PORTA_IRQHandler(void)
{
    uint32_t ISFR;
    /* safe copy */
    ISFR = PORT_InstanceTable[HW_GPIOA]->ISFR;
    /* clear IT pending bit */
    PORT_InstanceTable[HW_GPIOA]->ISFR = 0xFFFFFFFF;
    if(GPIO_CallBackTable[HW_GPIOA])
    {
        GPIO_CallBackTable[HW_GPIOA](ISFR);
    }	
}

void PORTB_IRQHandler(void)
{
    uint32_t ISFR;
    /* safe copy */
    ISFR = PORT_InstanceTable[HW_GPIOB]->ISFR;
    /* clear IT pending bit */
    PORT_InstanceTable[HW_GPIOB]->ISFR = 0xFFFFFFFF;
    if(GPIO_CallBackTable[HW_GPIOB])
    {
        GPIO_CallBackTable[HW_GPIOB](ISFR);
    }	
}

void PORTC_IRQHandler(void)
{
    uint32_t ISFR;
    /* safe copy */
    ISFR = PORT_InstanceTable[HW_GPIOC]->ISFR;
    /* clear IT pending bit */
    PORT_InstanceTable[HW_GPIOC]->ISFR = PORT_ISFR_ISF_MASK;
    if(GPIO_CallBackTable[HW_GPIOC])
    {
        GPIO_CallBackTable[HW_GPIOC](ISFR);
    }
}

void PORTD_IRQHandler(void)
{
    uint32_t ISFR;
    /* safe copy */
    ISFR = PORT_InstanceTable[HW_GPIOD]->ISFR;
    /* clear IT pending bit */
    PORT_InstanceTable[HW_GPIOD]->ISFR = PORT_ISFR_ISF_MASK;
    if(GPIO_CallBackTable[HW_GPIOD])
    {
        GPIO_CallBackTable[HW_GPIOD](ISFR);
    }	
}

void PORTE_IRQHandler(void)
{
    uint32_t ISFR;
    /* safe copy */
    ISFR = PORT_InstanceTable[HW_GPIOE]->ISFR;
    /* clear IT pending bit */
    PORT_InstanceTable[HW_GPIOE]->ISFR = PORT_ISFR_ISF_MASK;
    if(GPIO_CallBackTable[HW_GPIOE])
    {
        GPIO_CallBackTable[HW_GPIOE](ISFR);
    }	
}

#if (defined(MK70F12))
void PORTF_IRQHandler(void)
{
    uint32_t ISFR;
    /* safe copy */
    ISFR = PORT_InstanceTable[HW_GPIOF]->ISFR;
    /* clear IT pending bit */
    PORT_InstanceTable[HW_GPIOF]->ISFR = PORT_ISFR_ISF_MASK;
    if(GPIO_CallBackTable[HW_GPIOF])
    {
        GPIO_CallBackTable[HW_GPIOF](ISFR);
    }
}
#endif


//! @}
